/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/****  Project : eXtended C Library (XCLIB)
    *  Purpose : EXTRA MATHEMATIC FUNCTIONS
    *  Language: ANSI C
    *  File    : demo_add.c
    *  Author  : Martin Vicente <martin@vicente.org>
    *
    ****/


#define _XC_MATH_SOURCE

#include <errno.h>
#include <limits.h>
#include <float.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include <xc/mathdefs.h>
#include <xc/macros.h>
#include <xc/math.h>

static void demo_add(void);
static void demo_fadd(void);
static void print_fresult(double result);

#define COMM(en, fr)	__xcint(comm)(en, fr, NULL)
#define WAIT()		__xcint(wait)()

extern void __xcint(comm)(const char *en, ...);	/* DON'T USE IT! */
extern void __xcint(wait)(void);		/* DON'T USE IT! */

int main()
{
    demo_add();
    demo_fadd();

    return 0;
}

static void demo_add(void)
{
    int table1[] = { +INT_MAX, +INT_MAX };
    int table2[] = { +INT_MAX, +5, +INT_MAX, -1, -INT_MAX, +7, -INT_MAX };

    int result;

    COMM("Demonstration with `int'",
	 "Demonstration avec des `int'");

    result = add(table1, cardinal(table1));

    if ((result == INT_MIN || result == INT_MAX) && errno == ERANGE)
	puts(strerror(errno));
    else
	printf("result = %d\n", result);

    result = add(table2, cardinal(table2));

    if ((result == INT_MIN || result == INT_MAX) && errno == ERANGE)
	puts(strerror(errno));
    else
	printf("result = %d\n", result);

    WAIT();
}

static void demo_fadd(void)
{
    double table1[] = { +DBL_MAX, +DBL_MAX, PI, -DBL_MAX, -DBL_MAX };
    double table2[] = { +DBL_MAX, +DBL_MAX };

    double result;

    COMM("Demonstration with `double'",
	 "Demonstration avec des `double'");

    result = fadd(table1, cardinal(table1));

    if ((result == DBL_MIN || result == DBL_MAX) && errno == ERANGE)
	puts(strerror(errno));
    else
	printf("result = %g\n", result);

    result = fadd(table2, cardinal(table2));

    if ((result == DBL_MIN || result == DBL_MAX) && errno == ERANGE)
	puts(strerror(errno));
    else
	printf("result = %g\n", result);

    _math_flags = _mathFast;

    result = fadd(table2, cardinal(table2));

    if ((result == DBL_MIN || result == DBL_MAX) && errno == ERANGE)
	puts(strerror(errno));
    else
	print_fresult(result);

    WAIT();
}

static void print_fresult(double result)
{
    if (result > 0)
    {
	if (result == HUGE_VAL)
	    printf("result = Infinity\n");
	else if (result == DBL_MAX)
	    printf("result = DBL_MAX\n");
	else
	    printf("result = %g\n", result);
    }
    else if (result < 0)
    {
	if (result == -HUGE_VAL)
	    printf("result = -Infinity\n");
	else if (result == -DBL_MAX)
	    printf("result = -DBL_MAX\n");
	else
	    printf("result = %g\n", result);
    }
    else
    {
	printf("result = ZERO\n");
    }
}


/* End of file */
