/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/****  Project : eXtended C Library (XCLIB)
    *  Purpose : DYNAMIC MULTI-DIMENSIONAL ARRAY
    *  Language: ANSI C
    *  File    : demo_dynarr-1.c
    *  Author  : Martin Vicente <martin@vicente.org>
    *
    ****/


#include <stddef.h>
#include <stdio.h>

#include <xc/macros.h>
#include <xc/bool.h>
#include <xc/error.h>
#include <xc/dynarr.h>

static void demo1(void);
static void demo2(void);
static void demo3(void);
static void demo4(void);

#define COMM(en)	__xcint(comm)(en, NULL)
#define WAIT()		__xcint(wait)()

extern void __xcint(comm)(const char *en, ...);	/* DON'T USE IT!!! */
extern void __xcint(wait)(void);		/* DON'T USE IT!!! */

int main()
{
    demo1();
    demo2();
    demo3();
    demo4();

    return 0;
}

static void demo1(void)
{
    const int dim1 = 10;

    struct { int n, n2; } *arr;

    int i1;

    COMM("1D DYNAMIC ARRAY");

    arr = d1alloc(sizeof *arr, dim1);

    if (arr == NULL) error("Can't allocate memory");

    for (i1 = 0; i1 < dim1; i1++)
    {
	int n = i1 + 1;

	arr[i1].n  = n;
	arr[i1].n2 = SQR(n);
    }

    for (i1 = 0; i1 < dim1; ++i1)
    {
	printf("%d = %d  ", arr[i1].n, arr[i1].n2);
    }

    putchar('\n');

    dfree(arr);

    WAIT();
}

static void demo2(void)
{
    float cos_angle = 0.0;
    float sin_angle = 1.0;

    float **mat;

    COMM("2D DYNAMIC ARRAY");

    mat = d2alloc(sizeof (float), 2, 2);

    if (mat == NULL) error("Can't allocate memory");

    mat[0][0] = + cos_angle, mat[0][1] = + sin_angle;
    mat[1][0] = - sin_angle, mat[1][1] = + cos_angle;

    printf("|%+2g %+2g|\n", mat[0][0], mat[0][1]);
    printf("|%+2g %+2g|\n", mat[1][0], mat[1][1]);

    dfree(mat);

    WAIT();
}

static void demo3(void)
{
    const int dim1 = 3, dim2 = 5, dim3 = 5;

    bool ***arr;

    int i1, i2, i3;

    COMM("3D DYNAMIC ARRAY");

    arr = d3alloc(sizeof ***arr, dim1, dim2, dim3);

    if (arr == NULL) error("Can't allocate memory");

    for (i1 = 0; i1 < dim1; ++i1)
    {
	for (i2 = 0; i2 < dim2; ++i2)
	{
	    for (i3 = 0; i3 < dim3; ++i3)
	    {
		arr[i1][i2][i3] = i1 ^ i2 ^ i3;
	    }
	}
    }

    for (i1 = 0; i1 < dim1; ++i1)
    {
	printf("IMAGE %d:\n", i1);

	for (i2 = 0; i2 < dim2; ++i2)
	{
	    for (i3 = 0; i3 < dim3; ++i3)
	    {
		printf("%c ", arr[i1][i2][i3] ? '*' : ' ');
	    }

	    putchar('\n');
	}

	putchar('\n');
    }

    dfree(arr);

    WAIT();
}

static void demo4(void)
{
    const int dim1 = 2, dim2 = 3, dim3 = 4, dim4 = 7;

    struct { int i1, i2, i3, i4; } ****arr;

    int i1, i2, i3, i4;

    COMM("4D DYNAMIC ARRAY");

    arr = d4alloc(sizeof ****arr, dim1, dim2, dim3, dim4);

    if (arr == NULL) error("Can't allocate memory");

    for (i1 = 0; i1 < dim1; i1++)
    {
	for (i2 = 0; i2 < dim2; i2++)
	{
	    for (i3 = 0; i3 < dim3; i3++)
	    {
		for (i4 = 0; i4 < dim4; i4++)
		{
		    arr[i1][i2][i3][i4].i1 = i1;
		    arr[i1][i2][i3][i4].i2 = i2;
		    arr[i1][i2][i3][i4].i3 = i3;
		    arr[i1][i2][i3][i4].i4 = i4;
		}
	    }
	}
    }

    for (i1 = 0; i1 < dim1; i1++)
    {
	for (i2 = 0; i2 < dim2; i2++)
	{
	    for (i3 = 0; i3 < dim3; i3++)
	    {
		for (i4 = 0; i4 < dim4; i4++)
		{
		    printf(
			"%d%d%d%d ",
			arr[i1][i2][i3][i4].i1,
			arr[i1][i2][i3][i4].i2,
			arr[i1][i2][i3][i4].i3,
			arr[i1][i2][i3][i4].i4);
		}

		putchar('\n');
	    }

	    putchar('\n');
	}

	putchar('\n');
    }

    dfree(arr);

    WAIT();
}


/* End of file */
