/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/****  Project : eXtended C Library (XCLIB)
    *  Purpose : GENERIC LIST
    *  Language: ANSI C
    *  File    : demo_list-2.c
    *  Author  : Martin Vicente <martin@vicente.org>
    *
    ****/


/*****************************************************************************\

 [English]

 Purely generic use of <xc/list>.

 [French]

 Utilisation purement gnrique de <xc/list>.
  
\*****************************************************************************/


#include <stddef.h>
#include <stdio.h>

#include <xc/xalloc.h>
#include <xc/list.h>

#define PL	__xclib(LIST)

static void demo_numbers(void);
static void demo_strings(void);

static int put_number(Id id, void *data);
static int del_number(Id id, void *data);

static int put_string(Id id, void *data);
static int del_string(Id id, void *data);

#define COMM(en, fr)	__xcint(comm)(en, fr, NULL)
#define WAIT()		__xcint(wait)()

extern void __xcint(comm)(const char *en, ...);	/* DON'T USE IT!!! */
extern void __xcint(wait)(void);		/* DON'T USE IT!!! */

int main()
{
    demo_numbers();
    demo_strings();

    return 0;
}

static void demo_numbers(void)
{
    List list;

    COMM("Demonstration with numbers",
	 "Demonstration avec des nombres");

    list = PL.create(0);

    *(int *)PL.append(list, xnewrec(int))->item = 111;
    *(int *)PL.append(list, xnewrec(int))->item = 222;
    *(int *)PL.append(list, xnewrec(int))->item = 333;

    PL.action(list, put_number, NULL);

    PL.action(list, del_number, NULL);
    PL.destroy(list);

    WAIT();
}

static void demo_strings(void)
{
    List list;

    COMM("Demonstration with strings",
	 "Demonstration avec des chanes");

    list = PL.create(0);

    PL.append(list, xstrdup("AAA"));
    PL.append(list, xstrdup("BBB"));
    PL.append(list, xstrdup("CCC"));

    PL.action(list, put_string, NULL);

    PL.action(list, del_string, NULL);
    PL.destroy(list);

    WAIT();
}

static int put_number(Id id, __unused void *data)
{
    printf("\tnumber: %d\n", *(const int *)id->item);

    return 0;
}

static int del_number(Id id, __unused void *data)
{
    xfree((int *)id->item);

    return 0;
}

static int put_string(Id id, __unused void *data)
{
    printf("\tstring: %s\n", (char *)id->item);

    return 0;
}

static int del_string(Id id, __unused void *data)
{
    xfree((char *)id->item);

    return 0;
}


/* End of file */
