/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/****  Project : eXtended C Library (XCLIB)
    *  Purpose : PARSE COMMAND LINE OPTIONS
    *  Language: ANSI C
    *  File    : demo_options.c
    *  Author  : Martin Vicente <martin@vicente.org>
    *
    ****/


#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>

#include <xc/macros.h>
#include <xc/options.h>

struct MyData
{
    bool        verbose;
    int         count;
    const char *filename;
    float       factor;
    double      bigfactor;
    char        character;
    bool        help;
};

typedef struct MyData MyData;

static const Option options_desc[] =
{
#ifdef __GNUC__
    OPTION(MyData, help,      "--help",      NoArg,  Bool,   false,      true),
    OPTION(MyData, help,      "+h",          NoArg,  Bool,   false,      true),
    OPTION(MyData, verbose,   "--verbose",   NoArg,  Bool,   false,      true),
    OPTION(MyData, verbose,   "-v",          NoArg,  Bool,   false,      true),
    OPTION(MyData, count,     "--count",     SepArg, Int,    0,          0   ),
    OPTION(MyData, filename,  "--filename",  SepArg, String, "data.dat", NULL),
    OPTION(MyData, factor,    "--factor",    SepArg, Float,  1.21F,      0.0F),
    OPTION(MyData, bigfactor, "--bigfactor", SepArg, Double, 3.14,       0.0 ),
    OPTION(MyData, character, "--character", SepArg, Char,   'A',        '\0'),
#endif
};

static MyData myData;

int main(int argc, char *argv[])
{
    parse_options(&myData, options_desc, cardinal (options_desc), &argc, argv);

    if (myData.help)
    {
	printf(
	    "Usage:\n\n"
	    "\t-h, --help\n"
	    "\t-v, --verbose\n"
	    "\t    --count NUMBER\n"
	    "\t    --filename FILENAME\n"
	    "\t    --factor FLOAT\n"
	    "\t    --bigfactor DOUBLE\n"
	    "\t    --character CHAR\n");

	exit(0);
    }

    if (myData.verbose) puts("[VERBOSE MODE]");

    printf("count     = %d\n", myData.count);
    printf("factor    = %g\n", myData.factor);
    printf("bigfactor = %g\n", myData.bigfactor);
    printf("character = %c\n", myData.character);

    if (myData.filename != NULL) printf("filename  = %s\n", myData.filename);

    while (*argv != NULL) printf("# ARG --> %s\n", *argv++);

    return 0;
}


/* End of file */
