/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*  XCLIB 0.3	CLLIST:		CIRCULAR DOUBLY LINKED LIST		     */


#ifndef __XC_CLList_H
#define __XC_CLList_H


/* Standard C Library: */
#include <stddef.h>
#include <assert.h>

/* eXtended C Library: */
#include <xc/version.h>		/* VERSION OF EXTENDED C LIBRARY	     */


/*****************************************************************************
 * CUSTOMIZATION PART
 *****************************************************************************/


#ifndef CLL_NodeRec
#define CLL_NodeRec CLL_NodeRec
typedef struct node CLL_NodeRec;
#endif

#ifndef CLL_ListRec
#define CLL_ListRec CLL_ListRec
typedef struct list CLL_ListRec;
#endif

#ifndef CLL_Node
#define CLL_Node     CLL_Node
typedef CLL_NodeRec *CLL_Node;
#endif

#ifndef CLL_List
#define CLL_List     CLL_List
typedef CLL_ListRec *CLL_List;
#endif


#ifndef CLL_next
#define CLL_next	next
#endif

#ifndef CLL_prev
#define CLL_prev	prev
#endif

#ifndef CLL_link
#define CLL_link	link
#endif


/*****************************************************************************
 * INTERNALS
 *****************************************************************************/


#if defined __GNUC__ && !defined __STRICT_ANSI__
#define CLL_TYPEREC(NODEREC)	typeof (NODEREC)
#define CLL_TYPE(NODE)		typeof (NODE)
#else
#define CLL_TYPEREC(NODEREC)	CLL_NodeRec
#define CLL_TYPE(NODE)		CLL_Node
#endif


#define	__CLL_NEXT(NODE)	((NODE)->CLL_next)
#define	__CLL_PREV(NODE)	((NODE)->CLL_prev)
#define	__CLL_LINK(LIST)	((LIST)->CLL_link)


#define __CLL_FIRST(L)	(L)
#define __CLL_LAST(L)	__CLL_PREV(L)


#define __CLL_SANITY_CHECK(L) assert					      \
									      \
 ( (L) == NULL || (							      \
									      \
 __CLL_NEXT(L) != NULL &&						      \
 __CLL_PREV(L) != NULL ) )


#define _CLL_INIT(L)							      \
_MBEGIN									      \
    (L) = NULL;								      \
_MEND

#define _CLL_FOREACH_COND(L, NODE, COND)				      \
									      \
 for (									      \
									      \
 assert((L) != NULL), __CLL_SANITY_CHECK(L),				      \
									      \
 (NODE) = __CLL_NEXT(__CLL_FIRST(L)); (NODE) != (L) && (COND); (NODE) = __CLL_NEXT(NODE))

#define _CLL_FOREACH(L, NODE)	_CLL_FOREACH_COND(L, NODE, 1)

#define _CLL_FOREACH_NODE(L, INST)					      \
_MBEGIN									      \
    __XC_REGISTER CLL_TYPE(L) __node;					      \
									      \
    _CLL_FOREACH(L, __node) { INST }					      \
_MEND

#define _CLL_NEXT(N)	__CLL_NEXT(N)
#define _CLL_PREV(N)	__CLL_PREV(N)

#define _CLL_FIRST(L)	(__CLL_SANITY_CHECK(L), __CLL_FIRST(L))

#define _CLL_LAST(L)	(__CLL_SANITY_CHECK(L), ((L) != NULL ? __CLL_LAST(L) : NULL))

#define _CLL_APPEND(L, NODE)						      \
_MBEGIN									      \
    __CLL_SANITY_CHECK(L);						      \
									      \
    if ((L) != NULL)							      \
    {									      \
	_CLL_INSERT_BEFORE(NODE, L);					      \
    }									      \
    else								      \
    {									      \
	__CLL_NEXT(NODE) = (NODE);					      \
	__CLL_PREV(NODE) = (NODE);					      \
									      \
	(L) = (NODE);							      \
    }									      \
_MEND

#define _CLL_INSERT(L, NODE)						      \
_MBEGIN									      \
    __CLL_SANITY_CHECK(L);						      \
									      \
    if ((L) != NULL)							      \
    {									      \
	_CLL_INSERT_AFTER(NODE, L);					      \
    }									      \
    else								      \
    {									      \
	__CLL_NEXT(NODE) = (NODE);					      \
	__CLL_PREV(NODE) = (NODE);					      \
    }									      \
									      \
    (L) = (NODE);							      \
_MEND

#define _CLL_PREPEND(L, NODE) _CLL_INSERT(L, NODE)

#define _CLL_INSERT_BEFORE(NODE, MARK)					      \
_MBEGIN									      \
    assert((NODE) != NULL);						      \
    assert((MARK) != NULL);						      \
									      \
    __CLL_NEXT(NODE) = (MARK);						      \
    __CLL_PREV(NODE) = __CLL_PREV(MARK);				      \
									      \
    __CLL_PREV(__CLL_NEXT(NODE)) = (NODE);				      \
    __CLL_NEXT(__CLL_PREV(NODE)) = (NODE);				      \
_MEND

#define _CLL_INSERT_AFTER(NODE, MARK)					      \
_MBEGIN									      \
    assert((NODE) != NULL);						      \
    assert((MARK) != NULL);						      \
									      \
    __CLL_PREV(NODE) = (MARK);						      \
    __CLL_NEXT(NODE) = __CLL_NEXT(MARK);				      \
									      \
    __CLL_PREV(__CLL_NEXT(NODE)) = (NODE);				      \
    __CLL_NEXT(__CLL_PREV(NODE)) = (NODE);				      \
_MEND

#define __CLL_REMOVE(L, NODE)						      \
_MBEGIN									      \
    __CLL_SANITY_CHECK(L);						      \
									      \
    assert((NODE) != NULL);						      \
									      \
    if ((L) == (NODE)) (L) = __CLL_NEXT(NODE);				      \
    if ((L) == (NODE)) (L) = NULL;					      \
									      \
    __CLL_NEXT(__CLL_PREV(NODE)) = __CLL_NEXT(NODE);			      \
    __CLL_PREV(__CLL_NEXT(NODE)) = __CLL_PREV(NODE);			      \
									      \
    __CLL_NEXT(NODE) = NULL;						      \
    __CLL_PREV(NODE) = NULL;						      \
_MEND

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLL_REMOVE(L, NODE)	(__CLL_REMOVE(L, NODE), (NODE))
#else
#define _CLL_REMOVE(L, NODE)	__CLL_REMOVE(L, NODE)
#endif

#define _CLL_EXTRACT(L, NODE) _CLL_REMOVE(L, NODE)

#define _CLL_FOREWARD_ACTION(L, INST)					      \
_MBEGIN									      \
    __CLL_SANITY_CHECK(L);						      \
									      \
    if ((L) != NULL)							      \
    {									      \
	__XC_REGISTER CLL_TYPE(L) ___first_node, __node, ___next_node;	      \
									      \
	___first_node = __CLL_FIRST(L);					      \
									      \
	__node = ___first_node;						      \
									      \
	___next_node = __CLL_NEXT(__node);				      \
									      \
	INST								      \
									      \
	for (__node = ___next_node; __node != ___first_node; __node = ___next_node)\
	{								      \
	    ___next_node = __CLL_NEXT(__node);				      \
									      \
	    INST							      \
	}								      \
    }									      \
_MEND

#define _CLL_BACKWARD_ACTION(L, INST)					      \
_MBEGIN									      \
    __CLL_SANITY_CHECK(L);						      \
									      \
    if ((L) != NULL)							      \
    {									      \
	__XC_REGISTER CLL_TYPE(L) ___last_node, __node, ___prev_node;	      \
									      \
	___last_node = __CLL_LAST(L);					      \
									      \
	__node = ___last_node;						      \
									      \
	___prev_node = __CLL_PREV(__node);				      \
									      \
	INST								      \
									      \
	for (__node = ___prev_node; __node != ___last_node; __node = ___prev_node)\
	{								      \
	    ___prev_node = __CLL_PREV(__node);				      \
									      \
	    INST							      \
	}								      \
    }									      \
_MEND

#define _CLL_ACTION(L, INST)		_CLL_FOREWARD_ACTION(L, INST)
#define _CLL_FWD_ACTION(L, INST)	_CLL_FOREWARD_ACTION(L, INST)
#define _CLL_BWD_ACTION(L, INST)	_CLL_BACKWARD_ACTION(L, INST)

#define _CLL_CLEAN(L, INST)						      \
_MBEGIN									      \
    _CLL_ACTION(L, INST);						      \
									      \
    _CLL_INIT(L);							      \
_MEND

#define _CLL_PURGE(L, INST)	_CLL_PURGE(L, INST)

#define __CLL_LENGTH(L, LEN)						      \
_MBEGIN									      \
    __XC_REGISTER size_t __i = 0;					      \
									      \
    if ((L) != NULL) { __i++; _CLL_FOREACH_NODE(L, __i++;); }		      \
									      \
    (LEN) = __i;							      \
_MEND

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLL_LENGTH(L)							      \
_MBEGIN									      \
    __XC_REGISTER size_t __len;						      \
									      \
    __CLL_LENGTH(L, __len);						      \
									      \
    __len;								      \
_MEND
#else
#define _CLL_LENGTH(L, LEN)						      \
_MBEGIN									      \
    __XC_REGISTER size_t __len;						      \
									      \
    __CLL_LENGTH(L, __len);						      \
									      \
    (LEN) = __len;							      \
_MEND
#endif

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLL_PLENGTH(L, LEN)	((LEN) = _CLL_LENGTH(L))
#else
#define _CLL_PLENGTH(L, LEN)	_CLL_LENGTH(L, LEN)
#endif

#define __CLL_SELECT(L, INDEX, NODE)					      \
_MBEGIN									      \
    __XC_REGISTER size_t const __i = (INDEX);				      \
    __XC_REGISTER size_t       __j = 0;					      \
									      \
    if ((L) != NULL)							      \
    {									      \
	do								      \
	{								      \
	    (NODE) = __CLL_FIRST(L);					      \
									      \
	    if (__j++ == __i) break;					      \
									      \
	    _CLL_FOREACH(L, NODE) if (__j++ == __i) break;		      \
	}								      \
	while (0);							      \
    }									      \
    else								      \
    {									      \
	(NODE) = NULL;							      \
    }									      \
_MEND

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLL_SELECT(L, INDEX)						      \
_MBEGIN									      \
    __XC_REGISTER CLL_TYPE(L) __node;					      \
									      \
    __CLL_SELECT(L, INDEX, __node);					      \
									      \
    __node;								      \
_MEND
#else
#define _CLL_SELECT(L, INDEX, NODE)					      \
_MBEGIN									      \
    __XC_REGISTER CLL_TYPE(L) __node;					      \
									      \
    __CLL_SELECT(L, INDEX, __node);					      \
									      \
    (NODE) = __node;							      \
_MEND
#endif

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLL_PSELECT(L, INDEX, NODE)	((NODE) = _CLL_SELECT(L, INDEX))
#else
#define _CLL_PSELECT(L, INDEX, NODE)	_CLL_SELECT(L, INDEX, NODE)
#endif


/*****************************************************************************
 * HIGH LEVEL MACROS
 *****************************************************************************/


#define _CLL_LINK(LIST)	__CLL_LINK((assert((LIST) != NULL), LIST))


#define CLL_INIT(LIST)				_CLL_INIT(_CLL_LINK(LIST))
#define	CLL_NEXT(MARK)				_CLL_NEXT(MARK)
#define	CLL_PREV(MARK)				_CLL_PREV(MARK)
#define CLL_FIRST(LIST)				_CLL_FIRST(_CLL_LINK(LIST))
#define CLL_LAST(LIST)				_CLL_LAST(_CLL_LINK(LIST))
#define CLL_APPEND(LIST, NODE)			_CLL_APPEND(_CLL_LINK(LIST), NODE)
#define CLL_INSERT(LIST, NODE)			_CLL_INSERT(_CLL_LINK(LIST), NODE)
#define CLL_PREPEND(LIST, NODE)			_CLL_PREPEND(_CLL_LINK(LIST), NODE)
#define CLL_INSERT_BEFORE(NODE, MARK)		_CLL_INSERT_BEFORE(NODE, MARK)
#define CLL_INSERT_AFTER(NODE, MARK)		_CLL_INSERT_AFTER(NODE, MARK)
#define CLL_REMOVE(LIST, NODE)			_CLL_REMOVE(_CLL_LINK(LIST), NODE)
#define CLL_EXTRACT(LIST, NODE)			_CLL_EXTRACT(_CLL_LINK(LIST), NODE)
#define CLL_FOREACH_NODE(LIST, INST)		_CLL_FOREACH_NODE(_CLL_LINK(LIST), INST)
#define CLL_FOREACH(LIST, NODE)			_CLL_FOREACH(_CLL_LINK(LIST), NODE)
#define CLL_FOREACH_COND(LIST, NODE, COND)	_CLL_FOREACH_COND(_CLL_LINK(LIST), NODE, COND)
#define CLL_ACTION(LIST, INST)			_CLL_ACTION(_CLL_LINK(LIST), INST)
#define CLL_FWD_ACTION(LIST, INST)		_CLL_FWD_ACTION(_CLL_LINK(LIST), INST)
#define CLL_BWD_ACTION(LIST, INST)		_CLL_BWD_ACTION(_CLL_LINK(LIST), INST)
#define CLL_FOREWARD_ACTION(LIST, INST)		_CLL_FOREWARD_ACTION(_CLL_LINK(LIST), INST)
#define CLL_BACKWARD_ACTION(LIST, INST)		_CLL_BACKWARD_ACTION(_CLL_LINK(LIST), INST)
#define CLL_CLEAN(LIST, INST)			_CLL_CLEAN(_CLL_LINK(LIST), INST)
#define CLL_PURGE(LIST, INST)			_CLL_PURGE(_CLL_LINK(LIST), INST)

/*
 * Non portable versions
 */

#if defined __GNUC__ && !defined __STRICT_ANSI__

/* ====> Function behaviour */

#define CLL_LENGTH(LIST)			_CLL_LENGTH(_CLL_LINK(LIST))
#define CLL_SELECT(LIST, I)			_CLL_SELECT(_CLL_LINK(LIST), I)

#else

/* ====> strict ANSI conformance */

#define CLL_LENGTH(LIST, LEN)			_CLL_LENGTH(_CLL_LINK(LIST), LEN)
#define CLL_SELECT(LIST, I, NODE)		_CLL_SELECT(_CLL_LINK(LIST), I, NODE)

#endif

/*
 * Portable versions
 */

#define CLL_PLENGTH(LIST, LEN)			_CLL_PLENGTH(_CLL_LINK(LIST), LEN)
#define CLL_PSELECT(LIST, I, NODE)		_CLL_PSELECT(_CLL_LINK(LIST), I, NODE)


#endif /* XC CLList HEADER */


/*------------------------------/ END OF FILE /------------------------------*/
