/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*  XCLIB 0.3	CLLISTD:	CIRCULAR DOUBLY LINKED LIST WITH DUMMY NODE  */


#ifndef __XC_CLListD_H
#define __XC_CLListD_H


/* Standard C Library: */
#include <stddef.h>
#include <assert.h>

/* eXtended C Library: */
#include <xc/version.h>		/* VERSION OF EXTENDED C LIBRARY	     */


/*****************************************************************************
 * CUSTOMIZATION PART
 *****************************************************************************/


#ifndef CLLD_NodeRec
#define CLLD_NodeRec CLLD_NodeRec
typedef struct node  CLLD_NodeRec;
#endif

#ifndef CLLD_ListRec
#define CLLD_ListRec CLLD_ListRec
typedef struct list  CLLD_ListRec;
#endif

#ifndef CLLD_Node
#define CLLD_Node     CLLD_Node
typedef CLLD_NodeRec *CLLD_Node;
#endif

#ifndef CLLD_List
#define CLLD_List     CLLD_List
typedef CLLD_ListRec *CLLD_List;
#endif


#ifndef CLLD_next
#define CLLD_next	next
#endif

#ifndef CLLD_prev
#define CLLD_prev	prev
#endif

#ifndef CLLD_link
#define CLLD_link	link
#endif


/*****************************************************************************
 * INTERNALS
 *****************************************************************************/


#if defined __GNUC__ && !defined __STRICT_ANSI__
#define CLLD_TYPEREC(NODEREC)	typeof (NODEREC)
#define CLLD_TYPE(NODE)		typeof (NODE)
#else
#define CLLD_TYPEREC(NODEREC)	CLLD_NodeRec
#define CLLD_TYPE(NODE)		CLLD_Node
#endif


#define	__CLLD_NEXT(NODE)	((NODE)->CLLD_next)
#define	__CLLD_PREV(NODE)	((NODE)->CLLD_prev)
#define	__CLLD_LINK(LIST)	((LIST)->CLLD_link)


#define __CLLD_FIRST(L)	__CLLD_NEXT(L)
#define __CLLD_LAST(L)	__CLLD_PREV(L)


#define __CLLD_SANITY_CHECK(L) assert					      \
									      \
 ( (L) != NULL && (							      \
									      \
 __CLLD_NEXT(L) != NULL &&						      \
 __CLLD_PREV(L) != NULL ) )


#define _CLLD_INIT(L)							      \
_MBEGIN									      \
    assert((L) != NULL);						      \
									      \
    __CLLD_NEXT(L) = (L);						      \
    __CLLD_PREV(L) = (L);						      \
_MEND

#define _CLLD_FOREACH_COND(L, NODE, COND)				      \
									      \
 for (									      \
									      \
 __CLLD_SANITY_CHECK(L),						      \
									      \
 (NODE) = __CLLD_FIRST(L); (NODE) != (L) && (COND); (NODE) = __CLLD_NEXT(NODE))

#define _CLLD_FOREACH(L, NODE)	_CLLD_FOREACH_COND(L, NODE, 1)

#define _CLLD_FOREACH_NODE(L, INST)					      \
_MBEGIN									      \
    __XC_REGISTER CLLD_TYPE(L) __node;					      \
									      \
    _CLLD_FOREACH(L, __node) { INST }					      \
_MEND

#define _CLLD_NEXT(L, N)	(__CLLD_NEXT(N) != (L) ? __CLLD_NEXT(N) : NULL)
#define _CLLD_PREV(L, N)	(__CLLD_PREV(N) != (L) ? __CLLD_PREV(N) : NULL)

#define _CLLD_FIRST(L)							      \
									      \
 (__CLLD_SANITY_CHECK(L), __CLLD_FIRST(L) != (L) ? __CLLD_FIRST(L) : NULL)

#define _CLLD_LAST(L)							      \
									      \
 (__CLLD_SANITY_CHECK(L), __CLLD_LAST(L)  != (L) ? __CLLD_LAST(L) : NULL)

#define _CLLD_APPEND(L, NODE)						      \
_MBEGIN									      \
    __CLLD_SANITY_CHECK(L);						      \
									      \
    _CLLD_INSERT_BEFORE(NODE, L);					      \
_MEND

#define _CLLD_INSERT(L, NODE)						      \
_MBEGIN									      \
    __CLLD_SANITY_CHECK(L);						      \
									      \
    _CLLD_INSERT_AFTER(NODE, L);					      \
_MEND

#define _CLLD_PREPEND(L, NODE) _CLLD_INSERT(L, NODE)

#define _CLLD_INSERT_BEFORE(NODE, MARK)					      \
_MBEGIN									      \
    assert((NODE) != NULL);						      \
    assert((MARK) != NULL);						      \
									      \
    __CLLD_NEXT(NODE) = (MARK);						      \
    __CLLD_PREV(NODE) = __CLLD_PREV(MARK);				      \
									      \
    __CLLD_PREV(__CLLD_NEXT(NODE)) = (NODE);				      \
    __CLLD_NEXT(__CLLD_PREV(NODE)) = (NODE);				      \
_MEND

#define _CLLD_INSERT_AFTER(NODE, MARK)					      \
_MBEGIN									      \
    assert((NODE) != NULL);						      \
    assert((MARK) != NULL);						      \
									      \
    __CLLD_PREV(NODE) = (MARK);						      \
    __CLLD_NEXT(NODE) = __CLLD_NEXT(MARK);				      \
									      \
    __CLLD_PREV(__CLLD_NEXT(NODE)) = (NODE);				      \
    __CLLD_NEXT(__CLLD_PREV(NODE)) = (NODE);				      \
_MEND

#define __CLLD_REMOVE(NODE)						      \
_MBEGIN									      \
    assert((NODE) != NULL);						      \
									      \
    __CLLD_NEXT(__CLLD_PREV(NODE)) = __CLLD_NEXT(NODE);			      \
    __CLLD_PREV(__CLLD_NEXT(NODE)) = __CLLD_PREV(NODE);			      \
									      \
    __CLLD_NEXT(NODE) = NULL;						      \
    __CLLD_PREV(NODE) = NULL;						      \
_MEND

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLLD_REMOVE(NODE)	(__CLLD_REMOVE(NODE), (NODE))
#else
#define _CLLD_REMOVE(NODE)	__CLLD_REMOVE(NODE)
#endif

#define _CLLD_EXTRACT(NODE) _CLLD_REMOVE(NODE)

#define _CLLD_FOREWARD_ACTION(L, INST)					      \
_MBEGIN									      \
    __XC_REGISTER CLLD_TYPE(L) __node, ___next_node;			      \
									      \
    __CLLD_SANITY_CHECK(L);						      \
									      \
    for (__node = __CLLD_FIRST(L); __node != (L); __node = ___next_node)      \
    {									      \
	___next_node = __CLLD_NEXT(__node);				      \
									      \
	INST								      \
    }									      \
_MEND

#define _CLLD_BACKWARD_ACTION(L, INST)					      \
_MBEGIN									      \
    __XC_REGISTER CLLD_TYPE(L) __node, ___prev_node;			      \
									      \
    __CLLD_SANITY_CHECK(L);						      \
									      \
    for (__node = __CLLD_LAST(L); __node != (L); __node = ___prev_node)	      \
    {									      \
	___prev_node = __CLLD_PREV(__node);				      \
									      \
	INST								      \
    }									      \
_MEND

#define _CLLD_ACTION(L, INST)		_CLLD_FOREWARD_ACTION(L, INST)
#define _CLLD_FWD_ACTION(L, INST)	_CLLD_FOREWARD_ACTION(L, INST)
#define _CLLD_BWD_ACTION(L, INST)	_CLLD_BACKWARD_ACTION(L, INST)

#define _CLLD_CLEAN(L, INST)						      \
_MBEGIN									      \
    _CLLD_ACTION(L, INST);						      \
									      \
    _CLLD_INIT(L);							      \
_MEND

#define _CLLD_PURGE(L, INST)	_CLLD_PURGE(L, INST)

#define __CLLD_LENGTH(L, LEN)						      \
_MBEGIN									      \
    __XC_REGISTER size_t __i = 0;					      \
									      \
    _CLLD_FOREACH_NODE(L, __i++;);					      \
									      \
    (LEN) = __i;							      \
_MEND

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLLD_LENGTH(L)							      \
_MBEGIN									      \
    __XC_REGISTER size_t __len;						      \
									      \
    __CLLD_LENGTH(L, __len);						      \
									      \
    __len;								      \
_MEND
#else
#define _CLLD_LENGTH(L, LEN)						      \
_MBEGIN									      \
    __XC_REGISTER size_t __len;						      \
									      \
    __CLLD_LENGTH(L, __len);						      \
									      \
    (LEN) = __len;							      \
_MEND
#endif

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLLD_PLENGTH(L, LEN)	((LEN) = _CLLD_LENGTH(L))
#else
#define _CLLD_PLENGTH(L, LEN)	_CLLD_LENGTH(L, LEN)
#endif

#define __CLLD_SELECT(L, INDEX, NODE)					      \
_MBEGIN									      \
    __XC_REGISTER size_t const __i = (INDEX);				      \
    __XC_REGISTER size_t       __j = 0;					      \
									      \
    _CLLD_FOREACH(L, NODE) if (__j++ == __i) break;			      \
_MEND

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLLD_SELECT(L, INDEX)						      \
_MBEGIN									      \
    __XC_REGISTER CLLD_TYPE(L) __node;					      \
									      \
    __CLLD_SELECT(L, INDEX, __node);					      \
									      \
    __node;								      \
_MEND
#else
#define _CLLD_SELECT(L, INDEX, NODE)					      \
_MBEGIN									      \
    __XC_REGISTER CLLD_TYPE(L) __node;					      \
									      \
    __CLLD_SELECT(L, INDEX, __node);					      \
									      \
    (NODE) = __node;							      \
_MEND
#endif

#if defined __GNUC__ && !defined __STRICT_ANSI__
#define _CLLD_PSELECT(L, INDEX, NODE)	((NODE) = _CLLD_SELECT(L, INDEX))
#else
#define _CLLD_PSELECT(L, INDEX, NODE)	_CLLD_SELECT(L, INDEX, NODE)
#endif


/*****************************************************************************
 * HIGH LEVEL MACROS
 *****************************************************************************/


#define _CLLD_LINK(LIST)	__CLLD_LINK((assert((LIST) != NULL), LIST))


#define CLLD_INIT(LIST)				_CLLD_INIT(_CLLD_LINK(LIST))
#define CLLD_NEXT(LIST, MARK)			_CLLD_NEXT(_CLLD_LINK(LIST), MARK)
#define CLLD_PREV(LIST, MARK)			_CLLD_PREV(_CLLD_LINK(LIST), MARK)
#define CLLD_FIRST(LIST)			_CLLD_FIRST(_CLLD_LINK(LIST))
#define CLLD_LAST(LIST)				_CLLD_LAST(_CLLD_LINK(LIST))
#define CLLD_APPEND(LIST, NODE)			_CLLD_APPEND(_CLLD_LINK(LIST), NODE)
#define CLLD_INSERT(LIST, NODE)			_CLLD_INSERT(_CLLD_LINK(LIST), NODE)
#define CLLD_PREPEND(LIST, NODE)		_CLLD_PREPEND(_CLLD_LINK(LIST), NODE)
#define CLLD_INSERT_BEFORE(NODE, MARK)		_CLLD_INSERT_BEFORE(NODE, MARK)
#define CLLD_INSERT_AFTER(NODE, MARK)		_CLLD_INSERT_AFTER(NODE, MARK)
#define CLLD_REMOVE(NODE)			_CLLD_REMOVE(NODE)
#define CLLD_EXTRACT(NODE)			_CLLD_EXTRACT(NODE)
#define CLLD_FOREACH_NODE(LIST, INST)		_CLLD_FOREACH_NODE(_CLLD_LINK(LIST), INST)
#define CLLD_FOREACH(LIST, NODE)		_CLLD_FOREACH(_CLLD_LINK(LIST), NODE)
#define CLLD_FOREACH_COND(LIST, NODE, COND)	_CLLD_FOREACH_COND(_CLLD_LINK(LIST), NODE, COND)
#define CLLD_ACTION(LIST, INST)			_CLLD_ACTION(_CLLD_LINK(LIST), INST)
#define CLLD_FWD_ACTION(LIST, INST)		_CLLD_FWD_ACTION(_CLLD_LINK(LIST), INST)
#define CLLD_BWD_ACTION(LIST, INST)		_CLLD_BWD_ACTION(_CLLD_LINK(LIST), INST)
#define CLLD_FOREWARD_ACTION(LIST, INST)	_CLLD_FOREWARD_ACTION(_CLLD_LINK(LIST), INST)
#define CLLD_BACKWARD_ACTION(LIST, INST)	_CLLD_BACKWARD_ACTION(_CLLD_LINK(LIST), INST)
#define CLLD_CLEAN(LIST, INST)			_CLLD_CLEAN(_CLLD_LINK(LIST), INST)
#define CLLD_PURGE(LIST, INST)			_CLLD_PURGE(_CLLD_LINK(LIST), INST)

/*
 * Non portable versions
 */

#if defined __GNUC__ && !defined __STRICT_ANSI__

/* ====> Function behaviour */

#define CLLD_LENGTH(LIST)			_CLLD_LENGTH(_CLLD_LINK(LIST))
#define CLLD_SELECT(LIST, I)			_CLLD_SELECT(_CLLD_LINK(LIST), I)

#else

/* ====> strict ANSI conformance */

#define CLLD_LENGTH(LIST, LEN)			_CLLD_LENGTH(_CLLD_LINK(LIST), LEN)
#define CLLD_SELECT(LIST, I, NODE)		_CLLD_SELECT(_CLLD_LINK(LIST), I, NODE)

#endif

/*
 * Portable versions
 */

#define CLLD_PLENGTH(LIST, LEN)			_CLLD_PLENGTH(_CLLD_LINK(LIST), LEN)
#define CLLD_PSELECT(LIST, I, NODE)		_CLLD_PSELECT(_CLLD_LINK(LIST), I, NODE)


#endif /* XC CLListD HEADER */


/*------------------------------/ END OF FILE /------------------------------*/
