/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*  XCLIB 0.3	ERROR:		ERROR HANDLING FACILITY			     */


#ifndef __XC_ERROR_H
#define __XC_ERROR_H


/* Standard C Library: */
#include <stdarg.h>
#include <assert.h>

/* eXtended C Library: */
#include <xc/cdecl.h>		/* C DECLARATIONS			     */


#if	defined __DEBUG__

#ifdef __MACRO_VARARGS__
#define warning(fmt, arg...)		__xclib(dbg_warning)(__XC_ARGS, fmt, ##arg)
#define error(fmt, arg...)		__xclib(dbg_error)(__XC_ARGS, fmt, ##arg)
#define fatal(fmt, arg...)		__xclib(dbg_fatal)(__XC_ARGS, fmt, ##arg)
#define seterr(fmt, arg...)		__xclib(dbg_seterr)(__XC_ARGS, fmt, ##arg)
#define puterr(sev, fmt, arg...)	__xclib(dbg_puterr)(__XC_ARGS, sev, fmt, ##arg)
#else
#define warning(text)			__xclib(dbg_warning)(__XC_ARGS, text)
#define error(text)			__xclib(dbg_error)(__XC_ARGS, text)
#define fatal(text)			__xclib(dbg_fatal)(__XC_ARGS, text)
#define seterr(text)			__xclib(dbg_seterr)(__XC_ARGS, text)
#define puterr(sev, text)		__xclib(dbg_puterr)(__XC_ARGS, sev, text)
#endif
#define warning1(fmt, a1)		__xclib(dbg_warning)(__XC_ARGS, fmt, a1)
#define error1(fmt, a1)			__xclib(dbg_error)(__XC_ARGS, fmt, a1)
#define fatal1(fmt, a1)			__xclib(dbg_fatal)(__XC_ARGS, fmt, a1)
#define seterr1(fmt, a1)		__xclib(dbg_seterr)(__XC_ARGS, fmt, a1)
#define puterr1(sev, fmt, a1)		__xclib(dbg_puterr)(__XC_ARGS, sev, fmt, a1)
#define warning2(fmt, a1, a2)		__xclib(dbg_warning)(__XC_ARGS, fmt, a1, a2)
#define error2(fmt, a1, a2)		__xclib(dbg_error)(__XC_ARGS, fmt, a1, a2)
#define fatal2(fmt, a1, a2)		__xclib(dbg_fatal)(__XC_ARGS, fmt, a1, a2)
#define seterr2(fmt, a1, a2)		__xclib(dbg_seterr)(__XC_ARGS, fmt, a1, a2)
#define puterr2(sev, fmt, a1, a2)	__xclib(dbg_puterr)(__XC_ARGS, sev, fmt, a1, a2)
#define warning3(fmt, a1, a2, a3)	__xclib(dbg_warning)(__XC_ARGS, fmt, a1, a2, a3)
#define error3(fmt, a1, a2, a3)		__xclib(dbg_error)(__XC_ARGS, fmt, a1, a2, a3)
#define fatal3(fmt, a1, a2, a3)		__xclib(dbg_fatal)(__XC_ARGS, fmt, a1, a2, a3)
#define seterr3(fmt, a1, a2, a3)	__xclib(dbg_seterr)(__XC_ARGS, fmt, a1, a2, a3)
#define puterr3(sev, fmt, a1, a2, a3)	__xclib(dbg_puterr)(__XC_ARGS, sev, fmt, a1, a2, a3)
#define vseterr(fmt, va)		__xclib(dbg_vseterr)(__XC_ARGS, fmt, va)
#define vputerr(sev, fmt, va)		__xclib(dbg_vputerr)(__XC_ARGS, sev, fmt, va)

#elif	defined __FACI__

#ifdef __MACRO_VARARGS__
#define warning(fmt, arg...)		__xclib(f_warning)(__XC_FACI, fmt, ##arg)
#define error(fmt, arg...)		__xclib(f_error)(__XC_FACI, fmt, ##arg)
#define fatal(fmt, arg...)		__xclib(f_fatal)(__XC_FACI, fmt, ##arg)
#define seterr(fmt, arg...)		__xclib(f_seterr)(__XC_FACI, fmt, ##arg)
#define puterr(sev, fmt, arg...)	__xclib(f_puterr)(__XC_FACI, sev, fmt, ##arg)
#else
#define warning(text)			__xclib(f_warning)(__XC_FACI, text)
#define error(text)			__xclib(f_error)(__XC_FACI, text)
#define fatal(text)			__xclib(f_fatal)(__XC_FACI, text)
#define seterr(text)			__xclib(f_seterr)(__XC_FACI, text)
#define puterr(sev, text)		__xclib(f_puterr)(__XC_FACI, sev, text)
#endif
#define warning1(fmt, a1)		__xclib(f_warning)(__XC_FACI, fmt, a1)
#define error1(fmt, a1)			__xclib(f_error)(__XC_FACI, fmt, a1)
#define fatal1(fmt, a1)			__xclib(f_fatal)(__XC_FACI, fmt, a1)
#define seterr1(fmt, a1)		__xclib(f_seterr)(__XC_FACI, fmt, a1)
#define puterr1(sev, fmt, a1)		__xclib(f_puterr)(__XC_FACI, sev, fmt, a1)
#define warning2(fmt, a1, a2)		__xclib(f_warning)(__XC_FACI, fmt, a1, a2)
#define error2(fmt, a1, a2)		__xclib(f_error)(__XC_FACI, fmt, a1, a2)
#define fatal2(fmt, a1, a2)		__xclib(f_fatal)(__XC_FACI, fmt, a1, a2)
#define seterr2(fmt, a1, a2)		__xclib(f_seterr)(__XC_FACI, fmt, a1, a2)
#define puterr2(s, f, a1, a2)		__xclib(f_puterr)(__XC_FACI, sev, fmt, a1, a2)
#define warning3(fmt, a1, a2, a3)	__xclib(f_warning)(__XC_FACI, fmt, a1, a2, a3)
#define error3(fmt, a1, a2, a3)		__xclib(f_error)(__XC_FACI, fmt, a1, a2, a3)
#define fatal3(fmt, a1, a2, a3)		__xclib(f_fatal)(__XC_FACI, fmt, a1, a2, a3)
#define seterr3(fmt, a1, a2, a3)	__xclib(f_seterr)(__XC_FACI, fmt, a1, a2, a3)
#define puterr3(s, f, a1, a2, a3)	__xclib(f_puterr)(__XC_FACI, sev, fmt, a1, a2, a3)
#define vseterr(fmt, va)		__xclib(f_vseterr)(__XC_FACI, fmt, va)
#define vputerr(sev, fmt, va)		__xclib(f_vputerr)(__XC_FACI, sev, fmt, va)

#else

#ifdef __MACRO_VARARGS__
#define warning(fmt, arg...)		__xclib(warning)(fmt, ##arg)
#define error(fmt, arg...)		__xclib(error)(fmt, ##arg)
#define fatal(fmt, arg...)		__xclib(fatal)(fmt, ##arg)
#define seterr(fmt, arg...)		__xclib(seterr)(fmt, ##arg)
#define puterr(sev, fmt, arg...)	__xclib(puterr)(sev, fmt, ##arg)
#else
#define warning(text)			__xclib(warning)(text)
#define error(text)			__xclib(error)(text)
#define fatal(text)			__xclib(fatal)(text)
#define seterr(text)			__xclib(seterr)(text)
#define puterr(sev, text)		__xclib(puterr)(sev, text)
#endif
#define warning1(fmt, a1)		__xclib(warning)(fmt, a1)
#define error1(fmt, a1)			__xclib(error)(fmt, a1)
#define fatal1(fmt, a1)			__xclib(fatal)(fmt, a1)
#define seterr1(fmt, a1)		__xclib(seterr)(fmt, a1)
#define puterr1(sev, fmt, a1)		__xclib(puterr)(sev, fmt, a1)
#define warning2(fmt, a1, a2)		__xclib(warning)(fmt, a1, a2)
#define error2(fmt, a1, a2)		__xclib(error)(fmt, a1, a2)
#define fatal2(fmt, a1, a2)		__xclib(fatal)(fmt, a1, a2)
#define seterr2(fmt, a1, a2)		__xclib(seterr)(fmt, a1, a2)
#define puterr2(s, f, a1, a2)		__xclib(puterr)(sev, fmt, a1, a2)
#define warning3(fmt, a1, a2, a3)	__xclib(warning)(fmt, a1, a2, a3)
#define error3(fmt, a1, a2, a3)		__xclib(error)(fmt, a1, a2, a3)
#define fatal3(fmt, a1, a2, a3)		__xclib(fatal)(fmt, a1, a2, a3)
#define seterr3(fmt, a1, a2, a3)	__xclib(seterr)(fmt, a1, a2, a3)
#define puterr3(s, f, a1, a2, a3)	__xclib(puterr)(sev, fmt, a1, a2, a3)
#define vseterr(fmt, va)		__xclib(vseterr)(fmt, va)
#define vputerr(sev, fmt, va)		__xclib(vputerr)(sev, fmt, va)

#endif

#define seterrno(errnum)		__xclib(seterrno)(errnum)
#define puterrno(sev, errnum)		__xclib(puterrno)(sev, errnum)

#define clrerr()			__xclib(clrerr)()


_CDECL_BEGIN

void __xclib(warning)(const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 1, 2)));
void __xclib(error)(const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 1, 2), __noreturn__));
void __xclib(fatal)(const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 1, 2), __noreturn__));
void __xclib(seterr)(const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 1, 2)));
void __xclib(puterr)(const char *sev, const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 2, 3)));
void __xclib(vseterr)(const char *fmt, va_list)
 __attribute__ ((__format__ (__printf__, 1, 0)));
void __xclib(vputerr)(const char *sev, const char *fmt, va_list)
 __attribute__ ((__format__ (__printf__, 2, 0)));

void __xclib(f_warning)(__xc_faci, const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 2, 3)));
void __xclib(f_error)(__xc_faci, const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 2, 3), __noreturn__));
void __xclib(f_fatal)(__xc_faci, const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 2, 3), __noreturn__));
void __xclib(f_seterr)(__xc_faci, const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 2, 3)));
void __xclib(f_puterr)(__xc_faci, const char *sev, const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 3, 4)));
void __xclib(f_vseterr)(__xc_faci, const char *fmt, va_list)
 __attribute__ ((__format__ (__printf__, 2, 0)));
void __xclib(f_vputerr)(__xc_faci, const char *sev, const char *fmt, va_list)
 __attribute__ ((__format__ (__printf__, 3, 0)));

void __xclib(dbg_warning)(__xc_args, const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 4, 5)));
void __xclib(dbg_error)(__xc_args, const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 4, 5), __noreturn__));
void __xclib(dbg_fatal)(__xc_args, const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 4, 5), __noreturn__));
void __xclib(dbg_seterr)(__xc_args, const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 4, 5)));
void __xclib(dbg_puterr)(__xc_args, const char *sev, const char *fmt, ...)
 __attribute__ ((__format__ (__printf__, 5, 6)));
void __xclib(dbg_vseterr)(__xc_args, const char *fmt, va_list)
 __attribute__ ((__format__ (__printf__, 4, 0)));
void __xclib(dbg_vputerr)(__xc_args, const char *sev, const char *fmt, va_list)
 __attribute__ ((__format__ (__printf__, 5, 0)));

void __xclib(seterrno)(int errnum);
void __xclib(puterrno)(const char *severity, int errnum);

void __xclib(clrerr)(void);

_CDECL_END


#endif /* XC ERROR HEADER */


/*\
 *  ASSERT SECTION
\*/
#undef	assert
#ifdef	NDEBUG
#define	assert(expr)	__noop
#else
#ifndef	__FACI__
#undef	__XC_ARGS
#define __XC_ARGS	"", __FILE__, __LINE__
#endif
extern const char err_cassert[];
#define	assert(expr) ((void)((expr) ? 0 : (__xclib(dbg_fatal)(__XC_ARGS, err_cassert, #expr), 0)))
#endif
/*\
 *  END OF ASSERT SECTION
\*/


/*------------------------------/ END OF FILE /------------------------------*/
