/*  Copyright (C) 1996-2001  Martin Vicente			      -*- c -*-
    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*  XCLIB 0.3	ERRORS:		SOME COMMON ERRORS			     */


/*  !!! WARNING !!!    E X P E R I M E N T A L    !!! WARNING !!!  */


#ifndef __XC_ERRORS_H
#define __XC_ERRORS_H


#ifndef __EXPERIMENTAL__
#warning "<xc/errors.h> is experimental"
#endif


/* Standard C Library: */
#include <stdlib.h>

/* eXtended C Library: */
#include <xc/cdecl.h>		/* C DECLARATIONS			     */


/*********************************
 *          EXIT VALUES          *
 *********************************/


/* STANDARD EXIT CODES:
 */
#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS	0		/* ARGUMENT TO `exit(3)' */
#endif
#ifndef EXIT_FAILURE
#define EXIT_FAILURE	1		/* ARGUMENT TO `exit(3)' */
#endif

#if EXIT_SUCCESS != 0
#error "Value of `EXIT_SUCCESS' not equal to 0"
#endif

#if EXIT_FAILURE != 1
#error "Value of `EXIT_FAILURE' not equal to 1"
#endif

/* EXTENDED EXIT CODES: 2 <= code <= 255
 */
#define EXIT_MEMFULL	  2	/* Memory full				*/
#define EXIT_SYSFAIL	  3	/* System failure			*/
#define EXIT_SYSCALL	  4	/* System call error			*/
#define EXIT_CLIBERR	  5	/* C library function `%s()' fail	*/
#define EXIT_NORANGE	  6	/* Out of range				*/
#define EXIT_CONVERT	  7	/* Can not convert			*/
#define EXIT_IOERROR	  8	/* I/O error				*/
#define EXIT_NETWORK	  9	/* Network error			*/
#define EXIT_DISPLAY	 10	/* Can't open display "%s"		*/
#define EXIT_XWINDOW	 11	/* "X Window System" error		*/
#define EXIT_INVALID	 12	/* Invalid argument			*/
#define EXIT_INVALID	 12	/* Invalid argument			*/
#define EXIT_PROGRAM	 13	/* PROGRAM ERROR			*/


/*********************************
 *          ERROR TEXTS          *
 *********************************/


/*	ERROR		TEXT					   CODE */

#define ERR_SUCCESS	"SUCCESS"				/*    0 */
#define ERR_FAILURE	"FAILURE"				/*    1 */
#define ERR_MEMFULL	"Memory full"				/*    2 */
#define ERR_SYSFAIL	"System failure"			/*    3 */
#define ERR_SYSCALL	"System call error"			/*    4 */
#define ERR_CLIBERR	"C library function `%s()' fail"	/*    5 */
#define ERR_NORANGE	"Out of range"				/*    6 */
#define ERR_CONVERT	"Can not convert"			/*    7 */
#define ERR_IOERROR	"I/O error"				/*    8 */
#define ERR_NETWORK	"Network error"				/*    9 */
#define ERR_DISPLAY	"Can't open display \"%s\""		/*   10 */
#define ERR_XWINDOW	"\"X Window System\" error"		/*   11 */
#define ERR_INVALID	"Invalid argument"			/*   12 */

#define ERR_INVOCATION	"Invalid program invocation"		/*   12 */
#define ERR_INVOPT	"Unrecognized option"			/*   12 */
#define ERR_NOOPTARG	"No argument given for option `%s'"	/*   12 */
#define ERR_INVOPTARG	"Invalid option argument"		/*   12 */
#define ERR_PROGRAM	"PROGRAM ERROR"			     /* ABRT/13 */
#define ERR_INTERNAL	"INTERNAL ERROR"		     /* ABRT/13 */
#define ERR_CASSERT	"Assertion `%s' failed"		     /* ABRT/13 */
#define ERR_MAXARGS	"Maximum args count reached"	     /* ABRT/13 */

extern const char err_success[];
extern const char err_failure[];
extern const char err_memfull[];
extern const char err_sysfail[];
extern const char err_cliberr[];
extern const char err_norange[];
extern const char err_convert[];
extern const char err_ioerror[];
extern const char err_network[];
extern const char err_display[];
extern const char err_xwindow[];
extern const char err_invalid[];
extern const char err_program[];
extern const char err_cassert[];
extern const char err_maxargs[];


#define set_exit_status(text, code)	__xclib(set_exit_status)(text, code)
#define get_exit_status(text)		__xclib(get_exit_status)(text)


_CDECL_BEGIN

int __xclib(set_exit_status)(const char *text, int code);
int __xclib(get_exit_status)(const char *text);

_CDECL_END


#endif /* XC ERRORS HEADER */


/*------------------------------/ END OF FILE /------------------------------*/
