/*  XCLIB - a set of C functions for general purposes		      -*- c -*-
    Copyright (C) 1996-2001  Martin Vicente

    This file is part of the eXtended C library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*\
  |  Martin Vicente, 1996-2001
  |
  | Project : XCLIB (eXtended C Library)
  | Purpose : DUMMY FUNCTIONS
  | Version : 0.3
  | Language: ANSI C
  | File    : dummy.c
  | Author  : Martin VICENTE <martin@vicente.org>
\*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*/


#define _XCLIB_INTERNALS

#include <stddef.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include <xc/bool.h>

#include "global.h"

#define MSG_HIT_RETURN_EN	"Hit [Return] to continue"
#define MSG_HIT_RETURN_FR	"Tapez sur [Entre] pour continuer"

#define FORMAT	"\n\
######################################################################\n\
######################################################################\r\
###### %s%s%s \n\
######################################################################\n\
\n"

#define EQn(s1, s2)	(strncmp(s1, s2, strlen(s2)) == 0)

enum lang {en, fr};

static void initialize(void);

static enum lang lang;

static bool ready = false;

void __xcint(comm)(const char *comm_en, ...);
void __xcint(comm)(const char *comm_en, ...)
{
    const char bold[]     = "\x1b[1m";
    const char bold_off[] = "\x1b[22m";

    if (!ready) initialize();

    assert(comm_en != NULL);

    {
	const char *comm_fr = "";

	va_list     vargs;
	const char *comm;

	va_start(vargs, comm_en);
	if ((comm = va_arg(vargs, const char *)) != NULL) comm_fr = comm;
	va_end(vargs);

	switch (lang)
	{
	case en: comm = comm_en; break;
	case fr: comm = comm_fr; break;
	default: comm = comm_en; break;
	}

	if (comm[0] == '\0') comm = comm_en;

	printf(FORMAT, bold, comm, bold_off);
    }
}

void __xcint(wait)(void);
void __xcint(wait)(void)
{
    const char msg_en[] = MSG_HIT_RETURN_EN;
    const char msg_fr[] = MSG_HIT_RETURN_FR;

    if (!ready) initialize();

    {
	const char *msg;

	switch (lang)
	{
	case en: msg = msg_en; break;
	case fr: msg = msg_fr; break;
	default: msg = msg_en; break;
	}

	if (msg[0] == '\0') msg = msg_en;

	printf("\n%s", msg);
    }

    fflush(stdout);

    while (getchar() != '\n') ;
}

static void initialize(void)
{
    const char *const lang_env = getenv("LANG");

    if (lang_env == NULL)         lang = en;
    else if (EQn(lang_env, "fr")) lang = fr;
    else                          lang = en;

    ready = true;
}


/* End Of File */
