/*  XCLIB - a set of C functions for general purposes		      -*- c -*-
    Copyright (C) 1996-2001  Martin Vicente

    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*\
  |  Martin Vicente, 1996-2001
  |
  | Project : XCLIB (eXtended C Library)
  | Purpose : DYNAMIC MULTI-DIMENSIONAL ARRAY
  | Version : 0.3
  | Language: ANSI C
  | File    : dynarr/d3.c
  | Author  : Martin VICENTE <martin@vicente.org>
\*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*/


#define _XCLIB_INTERNALS

#define _FACILITY FACILITY(dynarr)

#include <stddef.h>
#include <stdlib.h>
#include <assert.h>

#include <xc/error.h>

#include "xc/dynarr.h"

#include "global.h"

#define NDIMS	3

typedef int DIM;

void *d3alloc(size_t const size, DIM const dim1, DIM const dim2, DIM const dim3)
{
    char   *sprite;
    char ***arr;

    register int i;

    assert(size > 0 && dim1 > 0 && dim2 > 0 && dim3 > 0);

    sprite = malloc(sizeof (int) + dim1 * sizeof (void **));

    if (sprite == NULL)
    {
	return NULL;
    }

    *(int *)sprite = NDIMS;

    arr = (char ***)(sprite + sizeof (int));

    arr[0] = malloc(dim1 * dim2 * sizeof (void *));

    if (arr[0] == NULL)
    {
	free(sprite);

	return NULL;
    }

    arr[0][0] = malloc(dim1 * dim2 * dim3 * size);

    if (arr[0][0] == NULL)
    {
	free(arr[0]);
	free(sprite);

	return NULL;
    }

    for (i = 1; i < dim1;        ++i) arr[i]    = arr[0]    + i * dim2;
    for (i = 1; i < dim1 * dim2; ++i) arr[0][i] = arr[0][0] + i * dim3 * size;

    return arr;
}

void d3free(void *const arr)
{
    char *sprite;

    assert(arr != NULL);

    sprite = (char *)arr - sizeof (int);

    assert(*(int *)sprite == NDIMS);

    {
	register char ***arr3D = arr;

	assert(arr3D && arr3D[0] && arr3D[0][0]);

	free(arr3D[0][0]);
	free(arr3D[0]);
	free(sprite);
    }
}


/* End Of File */
