/*  XCLIB - a set of C functions for general purposes		      -*- c -*-
    Copyright (C) 1996-2001  Martin Vicente

    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*\
  |  Martin Vicente, 1996-2001
  |
  | Project : XCLIB (eXtended C Library)
  | Purpose : ERROR HANDLING FACILITY (internals)
  | Version : 0.3
  | Language: ANSI C (with GNU things)
  | File    : error/intern.c
  | Author  : Martin VICENTE <martin@vicente.org>
\*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*/


#define _XCLIB_INTERNALS

#define _GNU_SOURCE

#include <limits.h>
#include <stddef.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include <xc/missing.h>
#include <xc/base.h>

#include "error/intern.h"

#include "global.h"

#define DELTA_CAPA	4

typedef const char *String;

typedef const String CString;

static void clear(void);

static struct errmsg *list     = NULL;
static size_t         count    = 0;
static size_t         capacity = 0;

void __xcint(error_set)(__xc_args, CString format, va_list const vargs)
{
    char *text;

    if (count == capacity)
    {
	struct errmsg *tmp;

	tmp = realloc(list, (capacity + DELTA_CAPA) * sizeof (struct errmsg));

	if (tmp == NULL)
	{
	    return;
	}

	list = tmp;

	capacity += DELTA_CAPA;
    }

    text = NULL;

    if (format != NULL)
    {
	if (vasprintf(&text, format, vargs) < 0 || text == NULL)
	{
	    return;
	}
    }

    list[count].faci = __faci;
    list[count].file = __file;
    list[count].line = __line;
    list[count].text = text;

    count++;
}

void __xcint(error_put)(__xc_args, CString severity, CString format, va_list const vargs)
{
    struct errmsg errmsg;

    register int i;

    errmsg.faci = __faci;
    errmsg.file = __file;
    errmsg.line = __line;
    errmsg.text = format;
    errmsg.next = count > 0 ? &list[count - 1] : NULL;

    assert(count <= INT_MAX);

    for (i = 0; i < (int)count; i++)
    {
	list[i].next = i > 0 ? &list[i - 1] : NULL;
    }

    puterrmsg(severity, &errmsg, vargs);

    if (count > 0) clear();
}

void __xcint(error_clr)(void)
{
    if (count > 0) clear();
}

static void clear(void)
{
    register int i;

    assert(count <= INT_MAX);

    for (i = 0; i < (int)count; i++)
	if (list[i].text != NULL)
	    free((char *)list[i].text);

    count = 0;
}


/* End Of File */
