/*  XCLIB - a set of C functions for general purposes		      -*- c -*-
    Copyright (C) 1996-2001  Martin Vicente

    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*\
  |  Martin Vicente, 1996-2001
  |
  | Project : XCLIB (eXtended C Library)
  | Purpose : SOME COMMON ERRORS
  | Version : 0.3
  | Language: ANSI C
  | File    : errors/exit_status.c
  | Author  : Martin VICENTE <martin@vicente.org>
\*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*/


/*****************************************************************************\

ATTENTION: ACTUELLEMENT NE FONCTIONNE QU'AVEC DES CHAINES LITTERALES.

D'ailleurs, est-il vraiment ncessaire de stocker ici les formats?

\*****************************************************************************/


#define _XCLIB_INTERNALS

#include <stddef.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include <xc/List.h>

#include "xc/errors.h"

#include "global.h"

#define EQn(txt, err)	(strncmp(txt, err, strlen(err)) == 0)

struct node
{
    struct node *next;
    const char  *text;
    int          code;
    unsigned int nref;
};

static struct node *list = NULL;

int __xclib(set_exit_status)(const char *text, int code)
{
    register struct node *node;

    assert(text != NULL);

    _L_FOREACH_COND(list, node, strcmp(node->text, text) != 0);

    if (code != 0)
    {
	if (node == NULL)
	{
	    node = malloc(sizeof(struct node));

	    if (node == NULL) return -1;

	    node->text = text;
	    node->code = 0;
	    node->nref = 0;

	    _L_APPEND(list, node);
	}

	node->code = code;

	node->nref++;
    }
    else
    {
	if (node == NULL) return -1;

	node->nref--;

	if (node->nref == 0)
	{
	    _L_REMOVE(list, node);

	    free(node);
	}
    }

    return 0;
}

int __xclib(get_exit_status)(const char *text)
{
    register struct node *node;

    _L_FOREACH_COND(list, node, strcmp(node->text, text) != 0);

    return node != NULL ? node->code : 0;
}


/* End Of File */
