/*  XCLIB - a set of C functions for general purposes		      -*- c -*-
    Copyright (C) 1996-2001  Martin Vicente

    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*\
  |  Martin Vicente, 1996-2001
  |
  | Project : XCLIB (eXtended C Library)
  | Purpose : SYNCHRONOUS I/O MULTIPLEXING (internals)
  | Version : 0.3
  | Language: ANSI C (with BSD things)
  | File    : io/ioset.c
  | Author  : Martin VICENTE <martin@vicente.org>
\*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*/


#define _XCLIB_INTERNALS

#define _BSD_SOURCE

#define _FACILITY	FACILITY(io)

#include <sys/types.h>

#include <stddef.h>
#include <assert.h>

#include "io/ioset.h"

#include "global.h"

#define AllBits	((1 << 0) | (1 << 1) | (1 << 2))

void __xcint(ioset_clr)(struct ioset *ioset)
{
    assert(ioset != NULL);

    FD_ZERO(&ioset->fdset[0]);
    FD_ZERO(&ioset->fdset[1]);
    FD_ZERO(&ioset->fdset[2]);

    ioset->fdmax = -1;
}

void __xcint(ioset_add)(struct ioset *ioset, int fd, unsigned int mask)
{
    assert(ioset != NULL);
    assert(fd >= 0 && fd < FD_SETSIZE);
    assert((mask & ~AllBits) == 0);

    if (mask &= AllBits)
    {
	if (mask & (1 << 0)) FD_SET(fd, &ioset->fdset[0]);
	if (mask & (1 << 1)) FD_SET(fd, &ioset->fdset[1]);
	if (mask & (1 << 2)) FD_SET(fd, &ioset->fdset[2]);

	if (fd > ioset->fdmax) ioset->fdmax = fd;
    }
}


/* END OF FILE */
