/*  XCLIB - a set of C functions for general purposes		      -*- c -*-
    Copyright (C) 1996-2001  Martin Vicente

    This file is part of the eXtended C Library (XCLIB).

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library (see the file COPYING.LIB).
    If not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.		     */


/*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*\
  |  Martin Vicente, 1996-2001
  |
  | Project : XCLIB (eXtended C Library)
  | Purpose : TRACE FACILITY (internals)
  | Version : 0.3
  | Language: ANSI C
  | File    : trace/intern.c
  | Author  : Martin VICENTE <martin@vicente.org>
\*|+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++|*/


/*****************************************************************************\

 [English]

 [French]

 Cette implmentation base sur alloca() permet d'assurer en partie l'atomicit
 de l'opration d'criture en effectuant un unique appel  une fonction de
 sortie de la bibliothque C: vfprintf().
 Par contre au niveau systme, rien ne garantit encore que la sortie se fera
 sans mlange avec un autre contexte d'excution. Pour pallier cette
 difficult, la version suivante utilisera en plus l'appel writev().

 Juste pour mmoire:  printf("%*s", INDENT * depth, "");

\*****************************************************************************/


#define _XCLIB_INTERNALS

#include "config.h"

#if !defined __GNUC__ || defined __STRICT_ANSI__
#  if !HAVE_ALLOCA_H
#    ifdef _AIX
#      pragma alloca
#    endif
#  endif
#endif

#include <sys/stat.h>

#include <limits.h>
#include <stddef.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>

#if !defined __GNUC__ || defined __STRICT_ANSI__
#  if HAVE_ALLOCA_H
#    include <alloca.h>
#  else
#    ifndef _AIX
#      ifndef alloca
         extern char *alloca();
#      endif
#    endif
#  endif
#endif

#include <xc/mathdefs.h>
#include <xc/colors.h>
#include <xc/bool.h>
#include <xc/program.h>
#include <xc/missing.h>
#include <xc/base.h>

#include "trace/intern.h"

#include "global.h"

#define INDENT	3

#define DEFAULT_OUTPUT	stdout

#define PREFIX_TRACE		"##"
#define PREFIX_TRACE_IN		"++"
#define PREFIX_TRACE_OUT	"--"
#define PREFIX_UNKNOWN		"??"

#define FG_COLOR_ENVIRONMENT	"TRACE_COLOR"
#define BG_COLOR_ENVIRONMENT	"TRACE_BG_COLOR"

#define FOREGROUND	1
#define BACKGROUND	0

#define STRINGSIZE(T)	(size_t)(sizeof (T) * CHAR_BIT * (M_LN2 / M_LN10) + 3)

#define TRACE_IS_ACTIVE	(_trace == _always || (_trace >= _auto && getenv("TRACE") != NULL))

#define __SET_STD_MSG()							      \
_MBEGIN									      \
    if (color)								      \
    {									      \
	set(foreground);						      \
									      \
	if (background != NULL)						      \
	{								      \
	    set(background);						      \
									      \
	    if (width > 0)						      \
	    {								      \
		register size_t i;					      \
									      \
		for (i = 0; i < width; i++) setc(' ');			      \
									      \
		setc('\r');						      \
	    }								      \
	}								      \
    }									      \
									      \
    if (program == NULL || program[0] != '\0')				      \
    {									      \
	if (program == NULL)						      \
	{								      \
	    set(null_string);						      \
	}								      \
	else								      \
	{								      \
	    set(program);						      \
	}								      \
									      \
	set(": ");							      \
    }									      \
									      \
    if (color) set(SGR_BOLD);						      \
									      \
    set(prefix);							      \
									      \
    if (depth > 0)							      \
    {									      \
	register int d, i;						      \
									      \
	if (color) set(SGR_BOLD_OFF);					      \
									      \
	for (d = 0; d < depth; ++d)					      \
	{								      \
	    set(" .");							      \
									      \
	    for (i = 2; i < INDENT; ++i) setc(' ');			      \
	}								      \
									      \
	if (color) set(SGR_BOLD);					      \
    }									      \
									      \
    if (faci == NULL || faci[0] != '\0')				      \
    {									      \
	setc(' ');							      \
									      \
	if (color) set(SGR_BOLD_OFF);					      \
	setc('[');							      \
	if (faci == NULL)						      \
	    set(null_string);						      \
	else								      \
	    set(faci);							      \
	setc(']');							      \
	if (color) set(SGR_BOLD);					      \
    }									      \
									      \
    if (format == NULL || format[0] != '\0')				      \
    {									      \
	setc(' ');							      \
									      \
	if (format == NULL)						      \
	    set(null_string);						      \
	else								      \
	    set(format);						      \
									      \
	if (type == TRACE_IN || type == TRACE_OUT)			      \
	{								      \
	    const char *pch;						      \
									      \
	    for (pch = format; *pch != '\0'; pch++)			      \
		;							      \
									      \
	    if ((pch - 1)[0] != ')')					      \
	    {								      \
		set("()");						      \
	    }								      \
	}								      \
    }									      \
									      \
    if (color) set(SGR_BOLD_OFF);					      \
									      \
    if (debug)								      \
    {	    								      \
	if (file == NULL || file[0] != '\0')				      \
	{								      \
	    set("\t- File ");						      \
									      \
	    if (file == NULL)						      \
		set(null_string);					      \
	    else							      \
	    {								      \
		setc('"');						      \
		if (color) set(SGR_BOLD);				      \
		set(file);						      \
		if (color) set(SGR_BOLD_OFF);				      \
		setc('"');						      \
	    }								      \
									      \
	    if (line > 0)						      \
	    {								      \
		char tmp[STRINGSIZE(int)];				      \
									      \
		if (snprintf(tmp, sizeof tmp, "%d", line) < 0) panic();	      \
									      \
		set(" at line ");					      \
		if (color) set(SGR_BOLD);				      \
		set(tmp);						      \
		if (color) set(SGR_BOLD_OFF);				      \
	    }								      \
	}								      \
    }									      \
									      \
    if (color) set(SGR_RESET);						      \
									      \
    setc('\n');								      \
_MEND

#define SET_STD_MSG()							      \
_MBEGIN									      \
    const String faci = info->faci;					      \
    const String file = info->file;					      \
    const int    line = info->line;					      \
									      \
    __SET_STD_MSG();							      \
_MEND

typedef const char *String;

typedef const String CString;

enum trace { TRACE, TRACE_IN, TRACE_OUT };

static void put(enum trace type, const struct info *info, CString format, va_list vargs);

static String setup_color(String *bg_color_return);

static int   depth  = 0;
static FILE *output = NULL;

void __xcint(trace)(const struct info *const info, CString format, va_list vargs)
{
    if (TRACE_IS_ACTIVE)
    {
	put(TRACE, info, format, vargs);
    }
}

void __xcint(trace_in)(const struct info *const info, CString function, va_list vargs)
{
    if (TRACE_IS_ACTIVE)
    {
	put(TRACE_IN, info, function, vargs);
    }

    ++depth;
}

void __xcint(trace_out)(const struct info *const info, CString function, va_list vargs)
{
    assert(depth > 0);

    --depth;

    if (TRACE_IS_ACTIVE)
    {
	put(TRACE_OUT, info, function, vargs);
    }
}

void __xcint(trace_set_output)(FILE *stream)
{
    assert(stream != NULL);

    output = stream;
}

static void put(enum trace type, const struct info *info, CString format, va_list vargs)
{
    const char null_string[] = NULL_STRING;

    const char prefix_trace[]     = PREFIX_TRACE;
    const char prefix_trace_in[]  = PREFIX_TRACE_IN;
    const char prefix_trace_out[] = PREFIX_TRACE_OUT;
    const char prefix_unknown[]   = PREFIX_UNKNOWN;

    bool   output_is_ok;
    bool   output_is_tty;
    bool   color;
    bool   debug;
    String foreground, background;
    size_t width;
    String program;
    String prefix;

    char *buffer;

    if (output == NULL) output = DEFAULT_OUTPUT;

    {
	struct stat status;

	output_is_ok = fstat(fileno(output), &status) != -1;

	output_is_tty = output_is_ok && S_ISCHR(status.st_mode);
    }

    color = _color == _always || (_color >= _auto && output_is_tty);

    if (color)
    {
	foreground = setup_color(&background);

	if (foreground == NULL)
	    color = false;
	else if (background != NULL)
	    width = 80;
    }

    debug = _debug == _always || (_debug >= _auto && getenv("DEBUG") != NULL);

    program = debug ? program_invocation_name : program_invocation_short_name;

    switch (type)
    {
    case TRACE:     prefix = prefix_trace;     break;
    case TRACE_IN:  prefix = prefix_trace_in;  break;
    case TRACE_OUT: prefix = prefix_trace_out; break;
    default:        prefix = prefix_unknown;   break;
    }

    {
	char ch[] = "?";

	size_t length = 0;

#define	set(s)	(length += strlen(s))
#define	setc(c)	(length += 1)
	SET_STD_MSG();
#undef	set
#undef	setc

	if ((buffer = alloca(length + 1)) == NULL) panic();

	buffer[0] = '\0';

#define	set(s)	strcat(buffer, s)
#define	setc(c)	(*ch = c, strcat(buffer, ch))
	SET_STD_MSG();
#undef	set
#undef	setc

	assert(strlen(buffer) == length);
    }

    if (fflush(output) == EOF) panic();

    if (fflush(NULL) == EOF) panic();

    if (vfprintf(output, buffer, vargs) < 0) panic();

    if (fflush(output) == EOF) panic();

#ifdef C_ALLOCA
    /* alloca(0); A N'UTILISER QUE DANS LE PROGRAMME!!! */
#endif
}

static String setup_color(String *bg_color_return)
{
    String foreground = NULL;
    String background = NULL;

    {
	String color_name;

	if ((color_name = getenv(FG_COLOR_ENVIRONMENT)) != NULL)
	    foreground = get_ansi_color_seq(color_name, FOREGROUND);

	if ((color_name = getenv(BG_COLOR_ENVIRONMENT)) != NULL)
	    background = get_ansi_color_seq(color_name, BACKGROUND);
    }

    if (bg_color_return != NULL) *bg_color_return = background;

    return foreground;
}


/* End Of File */
